Package: SMNlmec
Type: Package
Title: Scale Mixture of Normal Distribution in Linear Mixed-Effects
        Model
Version: 1.0.2
Authors@R: c(
  person("Kelin", "Zhong", email = "kelinzhonguconn@gmail.com", role = c("aut", "cre")),
  person("Fernanda L.", "Schumacher", email = "schumacher.313@osu.edu", role = c("aut")),
  person("Luis M.", "Castro", email = "lmcastro@uc.cl", role = c("aut")),
  person("Victor H.", "Lachos", email = "hlachos@uconn.edu", role = c("aut")),
  person("Jalmar M.F.", "Carrasco", email = "carrascojalmar@gmail.com", role = c("aut")))
Maintainer: Kelin Zhong <kelinzhonguconn@gmail.com>
Description: Bayesian analysis of censored linear mixed-effects models that replace Gaussian assumptions with a flexible class of distributions, such as the scale mixture of normal family distributions, considering a damped exponential correlation structure which was employed to account for within-subject autocorrelation among irregularly observed measures. For more details, see Kelin Zhong, Fernanda L. Schumacher, Luis M. Castro, Victor H. Lachos (2025) <doi:10.1002/sim.10295>.
Depends: R (>= 4.2)
Imports: rstan (>= 2.26.23), StanHeaders (>= 2.26.28), MASS (>=
        7.3-60), tmvtnorm (>= 1.5), mvtnorm (>= 1.2-3), mnormt (>=
        2.1.1), methods, stats, LaplacesDemon (>= 16.1.6),
        TruncatedNormal (>= 2.2.2), numDeriv (>= 2016.8-1.1)
URL: https://github.com/KelinZhong/SMNlmec
BugReports: https://github.com/KelinZhong/SMNlmec/issues
License: GPL-3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-05-02 21:51:19 UTC; Colin
Author: Kelin Zhong [aut, cre],
  Fernanda L. Schumacher [aut],
  Luis M. Castro [aut],
  Victor H. Lachos [aut],
  Jalmar M.F. Carrasco [aut]
Repository: CRAN
Date/Publication: 2026-05-02 22:10:02 UTC
Built: R 4.6.0; ; 2026-05-03 00:12:40 UTC; unix
