# Generated by roxygen2: do not edit by hand

export(SMNlmec.est)
export(SMNlmec.sim)
export(SMNlmec.summary)
exportClasses(SMNlmecfit)
import(MASS)
import(StanHeaders)
import(methods)
import(mnormt)
import(rstan)
import(stats)
import(tmvtnorm)
importFrom(LaplacesDemon,as.inverse)
importFrom(LaplacesDemon,dmvn)
importFrom(LaplacesDemon,is.positive.definite)
importFrom(LaplacesDemon,logdet)
importFrom(TruncatedNormal,mvNcdf)
importFrom(TruncatedNormal,mvNqmc)
importFrom(TruncatedNormal,pmvt)
importFrom(mvtnorm,dmvnorm)
importFrom(mvtnorm,dmvt)
importFrom(mvtnorm,pmvnorm)
importFrom(mvtnorm,rmvnorm)
importFrom(numDeriv,jacobian)
